@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2001 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,41 +15,36 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
  *                         videograbber.cpp  -  description
  *                         --------------------------------
  *   begin                : Mon Feb 12 2001
- *   copyright            : (C) 2000-2001 by Damien Sandras
+ *   copyright            : (C) 2000-2003 by Damien Sandras
  *   description          : Video4Linux compliant functions to manipulate the 
  *                          webcam device.
- *   email                : dsandras@seconix.com
  *
  */
 
 
 #include "../config.h"
 
-#include "config.h"
 #include "videograbber.h"
-#include "misc.h"
-#include "gdkvideoio.h"
-#include "common.h"
 #include "gnomemeeting.h"
-#include "dialog.h"
+#include "vfakeio.h"
 #include "misc.h"
 #include "menu.h"
-#include "vfakeio.h"
-
-
-#include <sys/time.h>
-#include <gconf/gconf-client.h>
-#ifndef DISABLE_GNOME
-#include <gnome.h>
-#else
-#include <gtk/gtk.h>
-#endif
+#include "dialog.h"
 
 
 /* Declarations */
@@ -58,23 +53,25 @@ extern GtkWidget *gm;
 
 
 /* The functions */
-GMVideoGrabber::GMVideoGrabber ()
-  :PThread (1000, NoAutoDeleteThread)
+GMVideoGrabber::GMVideoGrabber (PIntCondMutex *m,
+				BOOL start_grabbing,
+				BOOL sync)
+  :PThread (1000, AutoDeleteThread)
 {
   height = 0, width = 0;
   whiteness = 0, brightness = 0, colour = 0, contrast = 0;
 
-  gw = gnomemeeting_get_main_window (gm);
-  pw = gnomemeeting_get_pref_window (gm);
-  dw = gnomemeeting_get_druid_window (gm);
-
+  gw = MyApp->GetMainWindow ();
+  pw = MyApp->GetPrefWindow ();
+  dw = MyApp->GetDruidWindow ();
+  
   /* Internal state */
   is_running = 1;
-  has_to_open = 0;
   has_to_close = 0;
   has_to_reset = 0;
   has_to_stop = 0;
-  is_grabbing = 0;
+  is_grabbing = start_grabbing;
+  synchronous = sync;
   is_opened = 0;
 
   /* Initialisation */
@@ -90,91 +87,63 @@ GMVideoGrabber::GMVideoGrabber ()
   video_format = PVideoDevice::Auto;
   UpdateConfig ();
 
+  if (synchronous)
+    VGOpen ();
+
+  number_of_instances_mutex = m;
+
+  ++(*number_of_instances_mutex);
+  
   /* Start the thread */
   this->Resume ();
 }
 
 
 GMVideoGrabber::~GMVideoGrabber ()
-{ 
-  is_running = 0;
-  is_grabbing = 0;
+{
+  PWaitAndSignal m(device_mutex);
 
-  /* So that we can wait, in the Thread (Main ())
-     till the channel and grabber are closed, after the Main method
-     has exited */
-  quit_mutex.Wait ();
-
-
-  gnomemeeting_threads_enter ();
-
-  /* Disable the video settings frame */
-  gtk_widget_set_sensitive (GTK_WIDGET (gw->video_settings_frame),
-			    FALSE);
-
-  /* Enable the video preview buttons */
-  gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button),
-			    TRUE);
-#ifndef DISABLE_GNOME
-  gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), true);
-#endif
-  
-  /* Display the logo */
-  gnomemeeting_init_main_window_logo (gw->main_video_image);
-
-  gnomemeeting_threads_leave ();
-
-  g_free (video_device);
   g_free (color_format);
+  g_free (video_device);
+
+  --(*number_of_instances_mutex);
 }
 
 
 void GMVideoGrabber::Main ()
 {
+  static PMutex mutex;
+  PWaitAndSignal m(mutex);
+
+  
   /* We need those variables to facilitate the protection around mutexes */
-  int to_open, to_close, to_reset;
+  int to_reset = 0, to_run = 0;
 
-  /* Take the mutex, it will be released at the end of the method
-     because at the end of the method, the class can be deleted. */
-  quit_mutex.Wait ();
+  if (!synchronous)
+    VGOpen ();
 
-  while (is_running == 1) {
+  var_mutex.Wait ();
+  to_run = is_running;
+  var_mutex.Signal ();
+  
+  while (to_run == 1) {
 
     /* We protect variables that can be modified from several threads */
     var_mutex.Wait ();
-    to_open = has_to_open;
-    to_close = has_to_close;
     to_reset = has_to_reset;
+    to_run = is_running;
     var_mutex.Signal ();
 
-    if (to_open == 1) {
-     
-      UpdateConfig ();
-      VGOpen ();
-    }
-
-    if (to_close == 1) {
-
-      VGClose ();
-      UpdateConfig ();
-    }
-      
     /* The user can ask several resets */
     if (to_reset >= 1) {
 
-      if (IsOpened ()) {
-
- 	VGClose (0);
-	UpdateConfig ();
- 	VGOpen ();
-
-	var_mutex.Wait ();
- 	is_grabbing = 1;
-	var_mutex.Signal ();
-      }
-
+      VGClose ();
+      UpdateConfig ();
+      VGOpen ();
+      
       var_mutex.Wait ();
       has_to_reset--;
+      is_grabbing = 1;
       
       /* if we still need to reset more than one time, only reset one time */
       if (has_to_reset >= 1) 
@@ -184,40 +153,18 @@ void GMVideoGrabber::Main ()
 
     if (is_grabbing == 1) {
 
-      device_mutex.Wait ();
-      grabbing_mutex.Wait ();
-      
-      if (IsOpened ()) {
-
-	channel->Read (video_buffer, height * width * 3);
-	channel->Write (video_buffer, height * width * 3);
-      }
-    
-      grabbing_mutex.Signal ();
-      device_mutex.Signal ();
+      channel->Read (video_buffer, height * width * 3);
+      channel->Write (video_buffer, height * width * 3);    
     }
 
     Current()->Sleep (50);
   }
 
-  /* Disable the video preview button
-     It will be reenabled in the constructeur
-     that must be called just after the Main method exits. */
+  VGClose ();
+
   gnomemeeting_threads_enter ();
-  gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);
-#ifndef DISABLE_GNOME
-  gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), FALSE);
-#endif
+  gnomemeeting_init_main_window_logo (gw->main_video_image);
   gnomemeeting_threads_leave ();
-  
-  /* if opened, we close */
-  if (IsOpened ()) {
-
-    Current ()->Sleep (500);
-    delete (channel);
-  }
-
-  quit_mutex.Signal ();
 }
 
 
@@ -263,61 +210,11 @@ void GMVideoGrabber::UpdateConfig ()
 }
 
 
-void GMVideoGrabber::Open (int has_to_grab, int synchronous)
+void GMVideoGrabber::Close (void)
 {
-  int to_open;
-
-  /* Make sure to open with the right config */
-  UpdateConfig ();
-
   var_mutex.Wait ();
-  to_open = has_to_open;
+  is_running = 0;
   var_mutex.Signal ();
-
-  /* If the user asks for a synchronous call (typically, from
-     OpenVideoChannel), Open synchronously if an async opening
-     was not on the road. If an async opening was on the road,
-     then we wait till it is opened so that the user can be sure
-     that the device is opened when this function returns. */
-  if (synchronous == 1) {
-    if (to_open != 1)
-      VGOpen ();
-    else
-      while (!IsOpened ()) 
-	{PThread::Current ()->Sleep (100);}
-  }
-  else {
-
-    var_mutex.Wait ();
-    has_to_open = 1;
-    var_mutex.Signal ();
-  }
-    
-  var_mutex.Wait ();
-  is_grabbing = has_to_grab;
-  var_mutex.Signal ();
-}
-
-
-void GMVideoGrabber::Close (int synchronous)
-{
-  int to_close;
-
-  var_mutex.Wait ();
-  to_close = has_to_close;
-  var_mutex.Signal ();
-
-  if (synchronous == 1) {
-
-    if (to_close != 1)
-      VGClose (FALSE);
-  }
-  else {
-
-    var_mutex.Wait ();
-    has_to_close = 1;
-    var_mutex.Signal ();
-  }
 }
 
 
@@ -329,7 +226,7 @@ void GMVideoGrabber::Reset (void)
 }
 
 
-void GMVideoGrabber::Start (void)
+void GMVideoGrabber::StartGrabbing (void)
 {
   if (is_opened) {
 
@@ -340,7 +237,7 @@ void GMVideoGrabber::Start (void)
 }
 
 
-void GMVideoGrabber::Stop (void)
+void GMVideoGrabber::StopGrabbing (void)
 {
   var_mutex.Wait ();
   is_grabbing = 0;
@@ -348,27 +245,6 @@ void GMVideoGrabber::Stop (void)
 }
 
 
-void GMVideoGrabber::VGStop (void)
-{
-  var_mutex.Wait ();
-  is_grabbing = 0;
-  has_to_stop = 0;
-  var_mutex.Signal ();
-}
-
-
-int GMVideoGrabber::IsOpened (void)
-{
-  int is_open;
-
-  var_mutex.Wait ();
-  is_open = is_opened;
-  var_mutex.Signal ();
-  
-  return is_open;
-}
-
-
 GDKVideoOutputDevice *GMVideoGrabber::GetEncodingDevice (void)
 {
   return encoding_device;
@@ -381,15 +257,6 @@ PVideoChannel *GMVideoGrabber::GetVideoChannel (void)
 }
 
 
-void GMVideoGrabber::SetFrameRate (int fps)
-{
-  fps = PMAX (0, PMIN (fps, 30));
-
-  if (grabber != NULL)
-    grabber->SetFrameRate (fps);
-}
-
-
 void GMVideoGrabber::SetColour (int colour)
 {
   grabber->SetColour (colour);
@@ -418,7 +285,6 @@ void GMVideoGrabber::GetParameters (int *whiteness, int *brightness,
 				    int *colour, int *contrast)
 {
   int hue;
-
   grabber->GetParameters (whiteness, brightness, colour, contrast, &hue);
 
   *whiteness = (int) *whiteness / 256;
@@ -428,32 +294,28 @@ void GMVideoGrabber::GetParameters (int *whiteness, int *brightness,
 }
 
 
-void GMVideoGrabber::SetFrameSize (int height, int width)
+void GMVideoGrabber::Lock ()
 {
-  if ((encoding_device != NULL) && (grabber != NULL) && (is_opened)) {
-    
-    grabber->SetFrameSizeConverter (height, width, FALSE);
-    encoding_device->SetFrameSize (height, width);
-  }
+  device_mutex.Wait ();
 }
 
 
-void GMVideoGrabber::SetChannel (int cnum)
+void GMVideoGrabber::Unlock ()
 {
-  if (grabber != NULL)
-    grabber->SetChannel (cnum);
+  device_mutex.Signal ();
 }
 
 
 void GMVideoGrabber::VGOpen (void)
 {
+  gchar *title_msg = NULL;
   gchar *msg = NULL;
   gchar *video_image = NULL;
   int error_code = -1;  // No error
   int opened;
 
-  device_mutex.Wait ();
-
+  PWaitAndSignal m(device_mutex);
+  
   var_mutex.Wait ();
   opened = is_opened;
   var_mutex.Signal ();
@@ -470,9 +332,22 @@ void GMVideoGrabber::VGOpen (void)
     gnomemeeting_log_insert (gw->history_text_view, _("Opening Video device"));
     gnomemeeting_threads_leave ();
     
-    channel = new PVideoChannel ();
-    encoding_device = new GDKVideoOutputDevice (1, gw);
-    grabber = new PVideoInputDevice();
+#ifdef TRY_1394AVC
+    if (video_device == "/dev/raw1394" ||
+       strncmp (video_device, "/dev/video1394", 14) == 0) {
+            grabber = new PVideoInput1394AvcDevice();
+       }
+    else
+#endif
+#ifdef TRY_1394DC
+    if (video_device == "/dev/raw1394" ||
+        strncmp (video_device, "/dev/video1394", 14) == 0)
+           grabber = new PVideoInput1394DcDevice();
+    else
+#endif
+      {
+	 grabber = new PVideoInputDevice();
+      }
 
     if (video_size == 0) { 
       
@@ -496,7 +371,13 @@ void GMVideoGrabber::VGOpen (void)
       if (!grabber->Open (video_device, FALSE))
 	error_code = 0;
       else
-	if (!grabber->SetVideoChannelFormat (video_channel, video_format))
+	if (!grabber->SetFrameSizeConverter (width, height, FALSE))
+	  error_code = 5;
+      else
+	if (!grabber->SetVideoFormat(video_format))
+	  error_code = 2;
+      else
+        if (!grabber->SetChannel(video_channel))
           error_code = 2;
       else
 	if (!grabber->SetColourFormatConverter ("YUV420P"))
@@ -504,9 +385,6 @@ void GMVideoGrabber::VGOpen (void)
       else
 	if (!grabber->SetFrameRate (30))
 	  error_code = 4;
-      else
-	if (!grabber->SetFrameSizeConverter (width, height, FALSE))
-	   error_code = 5;
     }
 
 
@@ -530,40 +408,43 @@ void GMVideoGrabber::VGOpen (void)
       if (error_code != -2) {
 
 	gnomemeeting_threads_enter ();
-	msg = g_strdup_printf 
-	  (_("Error while opening video device %s, channel %d.\nThe chosen Video Image will be transmitted during calls. If you didn't choose any image, then the default GnomeMeeting logo will be transmitted. Notice that you can always transmit a given image or the GnomeMeeting logo by choosing \"Picture\" as video device."), video_device, video_channel);
+	title_msg =
+	  g_strdup_printf (_("Error while opening video device %s"), video_device);
+			   
+	msg = g_strdup (_("The chosen Video Image will be transmitted during calls. If you didn't choose any image, then the default GnomeMeeting logo will be transmitted. Notice that you can always transmit a given image or the GnomeMeeting logo by choosing \"Picture\" as video device."));
 	gnomemeeting_statusbar_flash (gw->statusbar, _("Can't open the Video Device"));
 	
 	switch (error_code)	{
 	  
 	case 0:
-	  msg = g_strconcat (msg, "\n", _("Error while opening the device."), 
+	  msg = g_strconcat (msg, "\n\n", _("There was an error while opening the device. Please check your permissions and make sure that the appropriate driver is loaded."), 
 			     NULL);
 	  break;
 	  
 	case 1:
-	  msg = g_strconcat (msg, "\n", _("Your video driver doesn't support the requested video format."), NULL);
+	  msg = g_strconcat (msg, "\n\n", _("Your video driver doesn't support the requested video format."), NULL);
 	  break;
 
 	case 2:
-	  msg = g_strconcat (msg, "\n", _("Could not open the chosen channel with the chosen video format."), NULL);
+	  msg = g_strconcat (msg, "\n\n", _("Could not open the chosen channel with the chosen video format."), NULL);
 	  break;
       
 	case 3:
-	  msg = g_strconcat (msg, "\n", g_strdup_printf(_("Your driver doesn't support the %s format.\n Please check your kernel driver documentation in order to determine which Palette is supported. Set it as GnomeMeeting default with:\n gconftool --set \"/apps/gnomemeeting/devices/color_format\" YOURPALETTE --type string"), color_format), NULL);
+	  msg = g_strconcat (msg, "\n\n", g_strdup_printf(_("Your driver doesn't seem to support any of the colour formats supported by GnomeMeeting.\n Please check your kernel driver documentation in order to determine which Palette is supported. Set it as GnomeMeeting default with:\n gconftool --set \"/apps/gnomemeeting/devices/color_format\" YOURPALETTE --type string")), NULL);
 	  break;
       
 	case 4:
-	  msg = g_strconcat (msg, "\n", _("Error with the frame rate."), NULL);
+	  msg = g_strconcat (msg, "\n\n", _("Error while setting the frame rate."), NULL);
 	  break;
 
 	case 5:
-	  msg = g_strconcat (msg, "\n", _("Error with the frame size."), NULL);
+	  msg = g_strconcat (msg, "\n\n", _("Error while setting the frame size."), NULL);
 	  break;
 	}
 
-	gnomemeeting_warning_dialog_on_widget (GTK_WINDOW (gm), gw->preview_button, msg);
+	gnomemeeting_warning_dialog_on_widget (GTK_WINDOW (gm), gw->preview_button, title_msg, msg);
 	g_free (msg);
+	g_free (title_msg);
 
 	gnomemeeting_threads_leave ();
       }
@@ -592,18 +473,22 @@ void GMVideoGrabber::VGOpen (void)
       g_free (video_image);
     }
 
+    
     grabber->Start ();
+
+    var_mutex.Wait ();
+    channel = new PVideoChannel ();
+    encoding_device = new GDKVideoOutputDevice (1, gw);
+    encoding_device->SetColourFormatConverter ("YUV420P");
+    var_mutex.Signal ();
     
     channel->AttachVideoReader (grabber);
     channel->AttachVideoPlayer (encoding_device);
 
-
     var_mutex.Wait ();
-    has_to_open = 0;
     is_opened = 1;
     var_mutex.Signal ();
   
-    /* If error */
     encoding_device->SetFrameSize (width, height);  
 
     /* Setup the video settings */
@@ -642,23 +527,19 @@ void GMVideoGrabber::VGOpen (void)
 
     gnomemeeting_threads_leave ();
   }  
-
-  device_mutex.Signal ();
 }
   
 
-void GMVideoGrabber::VGClose (int display_logo)
+void GMVideoGrabber::VGClose ()
 {
   int opened;
 
-  grabbing_mutex.Wait ();
-  device_mutex.Wait ();
+  PWaitAndSignal m(device_mutex);  
 
   var_mutex.Wait ();
   opened = is_opened;
   var_mutex.Signal ();
 
-
   if (opened) {
 
     var_mutex.Wait ();
@@ -690,11 +571,6 @@ void GMVideoGrabber::VGClose (int display_logo)
     gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), TRUE);
 #endif
     
-    /* Display the logo again */
-    if (display_logo)
-      gnomemeeting_init_main_window_logo (gw->main_video_image);
-
-
     gnomemeeting_statusbar_flash (gw->statusbar, _("Video Device Closed"));
     
     /* Disable the video settings frame */
@@ -717,16 +593,15 @@ void GMVideoGrabber::VGClose (int display_logo)
 
     /* Initialisation */
     grabber = NULL;
+    var_mutex.Wait ();
     channel = NULL;
     encoding_device = NULL;
+    var_mutex.Signal ();
   }
 
   /* Quick Hack for buggy drivers that return from the ioctl before the device
      is really closed */
   PThread::Current ()->Sleep (1000);
-
-  grabbing_mutex.Signal ();
-  device_mutex.Signal ();
 }
 
 
@@ -758,8 +633,7 @@ void GMVideoTester::Main ()
 
   GmWindow *gw = NULL;
   GmDruidWindow *dw = NULL;
-  
-  PVideoInputDevice *grabber = new PVideoInputDevice();
+  PVideoInputDevice *grabber = NULL;
   
   int height = GM_QCIF_HEIGHT; 
   int width = GM_QCIF_WIDTH; 
@@ -775,13 +649,29 @@ void GMVideoTester::Main ()
 
   video_device =  gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/devices/video_recorder", NULL);
 
-  gw = gnomemeeting_get_main_window (gm);
-  dw = gnomemeeting_get_druid_window (gm);
+  gw = MyApp->GetMainWindow ();
+  dw = MyApp->GetDruidWindow ();
   gtk_widget_set_sensitive (GTK_WIDGET (dw->video_test_button), FALSE);
   gtk_progress_bar_set_text (GTK_PROGRESS_BAR (dw->progress), "");
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (dw->progress), 0.0);
   gnomemeeting_threads_leave ();
 
+#ifdef TRY_1394AVC
+    if (video_device == "/dev/raw1394" ||
+       strncmp (video_device, "/dev/video1394", 14) == 0) {
+            grabber = new PVideoInput1394AvcDevice();
+       }
+    else
+#endif
+#ifdef TRY_1394DC
+    if (video_device == "/dev/raw1394" ||
+        strncmp (video_device, "/dev/video1394", 14) == 0)
+           grabber = new PVideoInput1394DcDevice();
+    else
+#endif
+      {
+	 grabber = new PVideoInputDevice();
+      }
   
   while (cpt <= 3) {
 
@@ -794,20 +684,20 @@ void GMVideoTester::Main ()
       
       if (!grabber->Open (video_device, FALSE))
 	error_code = 0;
+      else
+	if (!grabber->SetFrameSizeConverter (width, height, FALSE))
+	  error_code = 5;
       else
 	if (!grabber->SetVideoChannelFormat (0,  PVideoDevice::Auto))
 	  error_code = 2;
-	else
-	  if (!grabber->SetColourFormatConverter ("YUV420P"))
-	    error_code = 3;
-	  else
-	    if (!grabber->SetFrameRate (10))
-	      error_code = 4;
-	    else
-	      if (!grabber->SetFrameSizeConverter (width, height, FALSE))
-		error_code = 5;
-	      else
-		grabber->Close ();
+      else
+	if (!grabber->SetColourFormatConverter ("YUV420P"))
+	  error_code = 3;
+      else
+	if (!grabber->SetFrameRate (10))
+	  error_code = 4;
+      else
+	grabber->Close ();
 
       if (error_code != -1)
 	break;
